/*
 * (c) Copyright 1992 by Panagiotis Tsirigotis
 * All rights reserved.  The file named COPYRIGHT specifies the terms 
 * and conditions for redistribution.
 */


/*
 * $Id: sunos4.1-config.h,v 7.4 1992/06/22 20:34:42 panos Exp $
 */

/*
 * NOTE: ALL BINARY FLAGS MUST BE EITHER define'd OR undef'ed.
 *   		DO NOT SKIP FLAGS; undef THEM
 */


/************************ MEMORY MAPPING ****************************/

/*
 * Either define or undef MEMORY_MAP
 * If you define it, you must also define the memory mapping interface
 * functions: SIO_MMAP, SIO_MUNMAP, SIO_MNEED
 * SIO_MNEED is not necessary; if your OS does not support it
 * define it to nothing.
 */ 
#define MEMORY_MAP
#include <sys/types.h>
#include <sys/mman.h>

#define SIO_MMAP( addr, len, fd, off )												\
			mmap( addr, len, PROT_READ,												\
				( addr == 0 ) ? MAP_PRIVATE : MAP_PRIVATE + MAP_FIXED,		\
					fd, off )
#define SIO_MUNMAP( addr, len )	munmap( addr, len )
#define SIO_MNEED( addr, len )		(void) madvise( addr, len, MADV_WILLNEED ) 


/************************ FINALIZATION CODE ****************************/

/*
 * Either define or undef HAS_FINALIZATION
 * If you define it, you must define the macros SIO_FINALIZE 
 * and SIO_DEFINE_FIN
 * SIO_FINALIZE has a function as an argument and its value
 * is an expression whose value is TRUE if the finalization function
 * was successfully installed
 * SIO_DEFINE_FIN has a function as an argument and created an
 * appropriate function definition.
 */
#define HAS_FINALIZATION_FUNCTION
#define SIO_FINALIZE( func )			( on_exit( func, (caddr_t) 0 ) == 0 )
#define SIO_DEFINE_FIN( func )		static void func ( exit_status, arg ) 	\
														int exit_status ;						\
														caddr_t arg ;


/************************ GENERAL CONSTANTS ****************************/

/*
 * Define at most 1 of the following two flags
 */
#define HAS_MEMOPS
#undef HAS_BCOPY

/*
 * Define HAS_ISATTY if the function isatty( fd ) exists in the C library
 */
#define HAS_ISATTY

/*
 * N_SIO_DESCRIPTORS is the maximum number of file descriptors
 * supported by the OS
 */
#include <sys/param.h>
#define N_SIO_DESCRIPTORS			NOFILE

/************************ SPRINT CONSTANTS ****************************/

/*
 * SMALLEST_INT:     smallest integer the machine can represent
 * S_SMALLEST_INT:   the smallest integer as a string
 * S_SMALLEST_INT_LEN: length of string containing smallest integer
 *
 * These constants depend on the word size of the CPU and the
 * type of arithmetic (1's or 2's complement).
 */

#define SMALLEST_INT                (-2147483648)
#define S_SMALLEST_INT              "2147483648"      /* note: no '-' sign */
#define S_SMALLEST_INT_LEN          10

/*
 * Define LONG_IS_BIGGER is sizeof( long ) != sizeof( int )
 */
#undef LONG_IS_BIGGER

