/*
 * (c) Copyright 1992 by Panagiotis Tsirigotis
 * All rights reserved.  The file named COPYRIGHT specifies the terms 
 * and conditions for redistribution.
 */

static char RCSid[] = "$Id: xlog.c,v 1.4 1992/11/24 20:49:23 panos Exp $" ;
static char version[] = VERSION ;

#include <varargs.h>

#include "xlog.h"
#include "impl.h"

char *malloc() ;

extern struct xlog_ops __xlog_filelog_ops ;
#ifndef NO_SYSLOG
extern struct xlog_ops __xlog_syslog_ops ;
#endif

struct lookup_table
{
	struct xlog_ops *ops ;
	xlog_e type ;
} ;

static struct lookup_table ops_lookup_table[] =
	{
		{ &__xlog_filelog_ops,				XLOG_FILELOG	},
#ifndef NO_SYSLOG
		{ &__xlog_syslog_ops,				XLOG_SYSLOG		},
#endif
		{ NULL }
	} ;


#define CALLBACK( xp, status ) 																\
				if ( (xp)->callback ) 															\
					(*(xp)->callback)( (xlog_h)(xp), status, (xp)->callback_arg )


PRIVATE void xlog_link() ;
PRIVATE void xlog_unlink() ;
PRIVATE struct xlog_ops *xlog_ops_lookup() ;


/* VARARGS3 */
xlog_h xlog_create( type, id, flags, va_alist )
	xlog_e type ;
	char *id ;
	int flags ;
	va_dcl
{
	xlog_s *xp ;
	va_list ap ;
	struct xlog_ops *xops ;
	int status ;

	if ( ( xp = NEW( xlog_s ) ) == NULL )
		return( NULL ) ;
	
	if ( id == NULL || ( xp->id = __xlog_new_string( id ) ) == NULL )
	{
		FREE( xp ) ;
		return( NULL ) ;
	}

	xops = xlog_ops_lookup( type ) ;
	
	if ( xops != NULL )
	{
		va_start( ap ) ;
		status = (*xops->init)( xp, ap ) ;
		va_end( ap ) ;

		if ( status == XLOG_ENOERROR )
		{
			xp->active = TRUE ;
			xp->flags = flags ;
			xp->type = type ;
			xp->clients = XLOG_NULL ;
			xp->use = XLOG_NULL ;
			xp->ops = xops ;
			return( (xlog_h) xp ) ;
		}
	}

	free( xp->id ) ;
	FREE( xp ) ;
	return( NULL ) ;
}


PRIVATE struct xlog_ops *xlog_ops_lookup( type )
	register xlog_e type ;
{
	register struct lookup_table *ltp ;

	for ( ltp = &ops_lookup_table[ 0 ] ; ltp->ops ; ltp++ )
		if ( ltp->type == type )
			break ;
	return( ltp->ops ) ;
}



PRIVATE void xlog_link( client, server )
	xlog_s *client, *server ;
{
	client->use = server ;
	if ( server == NULL )
		return ;

	if ( server->clients == XLOG_NULL )
	{
		INIT_LINKS( client, other_users ) ;
		server->clients = client ;
	}
	else
		LINK( server, client, other_users ) ;
}


PRIVATE void xlog_unlink( xp )
	xlog_s *xp ;
{
	xlog_s *server = xp->use ;

	/*
	 * Step 1: remove from server chain
	 */
	if ( server != XLOG_NULL )
	{
		if ( server->clients == xp )
			if ( NEXT( xp, other_users ) == xp )
				server->clients = XLOG_NULL ;
			else
				server->clients = NEXT( xp, other_users ) ;
		else
			UNLINK( xp, other_users ) ;
	}

	/*
	 * Step 2: If we have users, clear their link to us.
	 */
	if ( xp->clients != NULL )
	{
		xlog_s *xp2 = xp->clients ;

		do
		{
			xp2->use = XLOG_NULL ;
			xp2 = NEXT( xp2, other_users ) ;
		}
		while ( xp2 != xp->clients ) ;
	}
}


PRIVATE void xlog_flags( xp, cmd, ap )
	xlog_s *xp ;
	xlog_cmd_e cmd ;
	va_list ap ;
{
	int flag = va_arg( ap, int ) ;
	int old_value = ( ( xp->flags & flag ) != 0 ) ;
	int *valp = va_arg( ap, int * ) ;

	if ( cmd == XLOG_SETFLAG )
		if ( *valp )
			xp->flags |= flag ;
		else
			xp->flags &= ~flag ;
		*valp = old_value ;
}


void xlog_destroy( xlog )
	xlog_h xlog ;
{
	xlog_s *xp = XP( xlog ) ;

	xlog_unlink( xp ) ;
	(*xp->ops->fini)( xp ) ;
	free( xp->id ) ;
	FREE( xp ) ;
}


/* VARARGS4 */
void xlog_write( xlog, buf, len, flags, va_alist )
	xlog_h xlog ;
	char buf[] ;
	int len ;
	int flags ;
	va_dcl
{
	xlog_s *xp = XP( xlog ) ;
	va_list ap ;
	int status ;

	if ( ! xp->active )
		return ;

	va_start( ap ) ;
	status = (*xp->ops->write)( xp, buf, len, flags, ap ) ;
	va_end( ap ) ;

	if ( status != XLOG_ENOERROR )
	{
		xp->active = FALSE ;
		CALLBACK( xp, status ) ;
	}
}


/* VARARGS2 */
void xlog_control( xlog, cmd, va_alist )
	xlog_h xlog ;
	xlog_cmd_e cmd ;
	va_dcl
{
	xlog_s *xp = XP( xlog ) ;
	va_list ap ;
	int status ;

	va_start( ap ) ;

	switch ( cmd )
	{
		case XLOG_LINK:
			xlog_unlink( xp ) ;
			xlog_link( xp, va_arg( ap, xlog_s * ) ) ;
			xp->callback_arg = va_arg( ap, void * ) ;
			break ;
		
		case XLOG_CALLBACK:
			xp->callback = va_arg( ap, voidfunc ) ;
			break ;
			
		case XLOG_GETFLAG:
		case XLOG_SETFLAG:
			xlog_flags( xp, cmd, ap ) ;
			status = XLOG_ENOERROR ;
			break ;

		default:
			status = (*xp->ops->control)( xp, cmd, ap ) ;
			if ( status == XLOG_ENOERROR )
				xp->active = TRUE ;
			else
			{
				xp->active = FALSE ;
				CALLBACK( xp, status ) ;
			}
	}

	va_end( ap ) ;
}


int xlog_parms( type, va_alist )
	xlog_e type ;
	va_dcl
{
	va_list ap ;
	int status ;

	va_start( ap ) ;
	if ( type == XLOG_SYSLOG )
		status = (*__xlog_syslog_ops.parms)( ap ) ;
	else if ( type == XLOG_FILELOG )
		status = (*__xlog_filelog_ops.parms)( ap ) ;
	else
		status = XLOG_ENOERROR ;
	va_end( ap ) ;
	return( status ) ;
}

